package common

import (
	"maps"
	"sync"
	"sync/atomic"
)

//********************************************************************************************************************************
// share lock (START)
//********************************************************************************************************************************

type ThreadSafeMap[K comparable, V any] struct {
	M map[K]V
	lock sync.RWMutex
}

/*
mp 可以为nil
*/
func (m *ThreadSafeMap[K,V]) Init(mp map[K]V, clone bool) *ThreadSafeMap[K,V] {
	if mp == nil {
		m.M = make(map[K]V)
	} else {
		if clone {
			m.M = maps.Clone(mp)
		} else {
			m.M = mp
		}
	}
	return m
}

func (m *ThreadSafeMap[K,V]) Get(k K) (v V, ok bool) {
	m.lock.RLock()
	v, ok = m.M[k]
	m.lock.RUnlock()
	return
}

func (m *ThreadSafeMap[K,V]) Contain(k K) (ok bool) {
	m.lock.RLock()
	_, ok = m.M[k]
	m.lock.RUnlock()
	return
}

func (m *ThreadSafeMap[K,V]) Set(k K, v V) {
	m.lock.Lock()
	m.M[k] = v
	m.lock.Unlock()
}

func (m *ThreadSafeMap[K,V]) Do(fn func(map[K]V)) {
	m.lock.Lock()
	fn(m.M)
	m.lock.Unlock()
}

// only replace exist
func (m *ThreadSafeMap[K,V]) ReplaceIfExist(k K, v V) V {
	m.lock.Lock()
	old := m.M[k]
	m.M[k] = v
	m.lock.Unlock()
	return old
}

func (m *ThreadSafeMap[K,V]) Delete(k K) {
	m.lock.Lock()
	delete(m.M, k)
	m.lock.Unlock()
}

func (m *ThreadSafeMap[K,V]) Size() int {
	m.lock.RLock()
	ln := len(m.M)
	m.lock.RUnlock()
	return ln
}

func (m *ThreadSafeMap[K,V]) Rest() map[K]V {
	m.lock.Lock()
	defer m.lock.Unlock()
	old := m.M
	m.M = make(map[K]V)
	return old 
}

func (m *ThreadSafeMap[K,V]) Run(fn func(map[K]V) error) error {
	m.lock.Lock()
	defer m.lock.Unlock()
	return fn(m.M)
}

/*
	遍历 m, fn 返回 false 结束
*/
func (m *ThreadSafeMap[K,V]) Traverse(fn func(K,V) bool) {
	m.lock.Lock()
	defer m.lock.Unlock()
	for k, v := range m.M {
		if !fn(k, v) { return }
	}
}

func (m *ThreadSafeMap[K,V]) Lock() { m.lock.Lock() }
func (m *ThreadSafeMap[K,V]) UnLock() { m.lock.Unlock() }

//********************************************************************************************************************************
// share lock (END)
//********************************************************************************************************************************

//********************************************************************************************************************************
// atomic (START)
//********************************************************************************************************************************
type AtomicMap[K comparable, V any] map[K]*atomic.Pointer[V]

func NewAtomicMap[K comparable, V any]() AtomicMap[K, V] {
	return make(map[K]*atomic.Pointer[V])
}

func (m AtomicMap[K, V]) Add(k K, ptrV *V) {
	if a, ok := m[k]; ok {
		a.Store(ptrV)
	} else {
		a = new(atomic.Pointer[V])
		a.Store(ptrV)
		m[k] = a
	}
}

func (m AtomicMap[K, V]) Get(k K) *V {
	if a, ok := m[k]; ok {
		return a.Load()
	} else {
		return nil
	}
}
//********************************************************************************************************************************
// atomic (END)
//********************************************************************************************************************************